@@ -1,5 +1,5 @@
 from re import split, sub
-from typing import DefaultDict, Dict, Union, TextIO, List, Any, Generator, Tuple, Iterable, Mapping, Hashable
+from typing import DefaultDict, Dict, Union, TextIO, List, Any, Generator, Tuple, Iterable, Mapping, Hashable, Type
 from sys import argv, version_info
 from os import getenv, name as os_name, environ as os_environ
 if os_name == 'nt':
@@ -7,6 +7,7 @@ if os_name == 'nt':
 from io import StringIO, BytesIO
 from collections import defaultdict, OrderedDict
 from pathlib import Path
+from copy import copy
 
 GLOBALS_KEY = '_globals'
 
@@ -354,8 +355,23 @@ class DictConfig(dict):
         :param default: default to return if key is not found
         :return: self[key]
         """
-        result = self.__getitem__(key)
-        return default if key not in self or result is None else result
+        if key not in self:
+            return default
+        try:
+            result = self.__getitem__(key)
+            if result is None:
+                return default
+            return result
+        except KeyError:
+            return default
+
+    def get_as_type(self, key: Hashable, as_type: Type, default: Any = None) -> Any:
+        value = self.get(key, default)
+        if value is None:
+            return None
+        if as_type is bool and isinstance(value, str):
+                return value.lower() not in ['0', 'f', 'false']
+        return as_type(value)
 
     def pop(self, key: Hashable, default: Any = None) -> Any:
         """
@@ -780,17 +796,17 @@ class DictConfig(dict):
         if not isinstance(cli_args, dict):
             arg_map = cls.ARG_MAP
             if aliases is not None:
-                arg_map.update(cls.ARG_MAP)
+                arg_map.update(aliases)
             return argv_to_dict(cli_args if isinstance(cli_args, list) else argv, arg_map)
         if isinstance(cli_args, defaultdict):
             # noinspection PyArgumentList
             return cli_args.__class__(
                 cli_args.default_factory,
-                {aliases[k] if aliases is not None and k in aliases else k: v for k, v in cli_args.items()})
+                {aliases[k] if aliases is not None and k in aliases else k: copy(v) for k, v in cli_args.items()})
         else:
             # noinspection PyArgumentList
             return cli_args.__class__(
-                {aliases[k] if aliases is not None and k in aliases else k: v for k, v in cli_args.items()})
+                {aliases[k] if aliases is not None and k in aliases else k: copy(v) for k, v in cli_args.items()})
 
     def parse_arguments(self, cli_args: Union[Dict[str, list], list] = None, aliases: Dict[str, str] = None):
         """
@@ -916,13 +932,19 @@ class DictConfig(dict):
         :return: None
         """
         # is k an existing key?
+        excess = []
         if k in self:
             # for bool, check specific non-True values
             if isinstance(self[k], bool):
-                self[k] = v.lower() not in ['0', 'false']
+                self[k] = v if isinstance(v, bool) else v.lower() not in ['0', 'false']
             else:
                 # for other types, cast to type of existing key
                 t = type(self[k])
+                if isinstance(v, (list, tuple)) and (t not in (list, tuple)):
+                    excess = list(v[1:])
+                    v = v[0]
+                if (t in (list, tuple)) and not isinstance(v, (list, tuple)):
+                    v = [v]
                 try:
                     self[k] = t(v)
                 except ValueError:
@@ -931,6 +953,7 @@ class DictConfig(dict):
             # define new key
             self.from_arguments.append(k)
             self[k] = v
+        return excess
 
     def _set_globals_from_args(self, k, v):
         """
@@ -939,18 +962,28 @@ class DictConfig(dict):
         :param v: value to set
         :return: None
         """
+        excess = []
         if self.globals is None:
             self.globals = {}
         # for bool, check specific non-True values
         if k in self.globals and isinstance(self.globals[k], bool):
-            self.globals[k] = v.lower() not in ['0', 'false']
+            self.globals[k] = v if isinstance(v, bool) else v.lower() not in ['0', 'false']
         else:
-            # for other types, cast to type of existing key
-            t = type(self.globals[k]) if k in self.globals else type(v)
+            # for other types, cast to type of existing key and remove excess values for non-iterable predefined values
+            if k in self.globals:
+                t = type(self.globals[k])
+                if isinstance(v, (list, tuple)) and (t not in (list, tuple)):
+                    excess = list(v[1:])
+                    v = v[0]
+                if (t in (list, tuple)) and not isinstance(v, (list, tuple)):
+                    v = [v]
+            else:
+                t = type(v)
             try:
                 self.globals[k] = t(v)
             except ValueError:
                 raise SyntaxError('Cannot cast {} to {} from arguments'.format(v, t))
+        return excess
 
     def update_from_arguments(self, cli_args: Union[Dict[str, list], list] = None, aliases: Dict[str, str] = None):
         """
@@ -978,13 +1011,19 @@ class DictConfig(dict):
                     update = self._set_value_from_args
                 # unpack single value lists
                 if len(value) == 1:
-                    update(key, value[0])
+                    excess = update(key, value[0])
                 elif not value:
                     # set to True for empty value
-                    update(key, True)
+                    excess = update(key, True)
                 else:
                     # set as list for multi-value
-                    update(key, value)
+                    excess = update(key, value)
+                if excess:
+                    self.arguments[''].extend(excess)
+                    self.arguments[key] = value[0]
+                    if cli_args is not None:
+                        cli_args[''].extend(excess)
+                        cli_args[key] = value[0]
 
         # allow chaining
         return self
@@ -1029,7 +1068,7 @@ class DictConfig(dict):
         elif defaults is None:
             return cls.load(require_file=False, **kwargs).full_update(**fu_kwargs)
         else:
-            return cls(defaults).update(
+            return cls(defaults, **kwargs).update(
                 cls.load(require_file=False, **kwargs)
             ).full_update(**fu_kwargs)
 
